import pandas as pd
from matplotlib import pyplot as plt

def named(df, name):
     named.register.append((name, df))
     return df
     
named.register = []

def save_named(name):
    for name_, obj in named.register:
        if name == name_:
            obj.to_csv(name + '.csv')
            print('Saved ' + name + ':')
            print(obj.head())
            
def get_named(name):
    return named.register[name]
            
            
def plot_named(name, x, ys, *args, **kwargs):
    for name_, obj in named.register:
        if name == name_:
            cols = obj.columns
            
            for y in ys:
                plt.plot(getattr(obj, x), getattr(obj, y), *args, **kwargs)
                
            plt.xlabel(x)
            plt.ylabel(', '.join(ys))
            plt.savefig('images/' + name)
            plt.show()
            
            
def show_image(name):
    plt.imshow(plt.imread('images/' + name + '.png'))
    plt.box(on=False)
    plt.show()
    
def make_title(author=None, date=None, title=None):
    text = ''
    if author is not None:
        text += '\\author{' + author + '}\n'

    if date is not None:
        if date == 'today':
            text += '\\date{\\today}\n'
        else:
            text += '\\date{' + date + '}\n'
            
    if title is not None:
        text += '\\title{' + title + '}\n'
    
    with open('TITLE_TEXT.txt', 'w') as O:
        O.write(text)



preambule = r"""

\documentclass[a4paper,12pt]{article}

\usepackage[T2A]{fontenc}
\usepackage[utf8]{inputenc}
\usepackage[english,russian]{babel}
\usepackage[pdftex]{graphicx}
\usepackage{array,tabularx,tabulary,booktabs}
\usepackage{longtable}  

\graphicspath{{images/}}



\usepackage{amsmath,amsfonts,amssymb,amsthm,mathtools} 
"""

term = r"""

\end{document} 
"""

head_end = r'''
\begin{document}

\maketitle'''



from setuptools import setup, find_packages 
import os

os.system('rm -rf dist')

setup(name='excelmustdie',
version='0.4',
description='A MIPT lab report creation tool',
long_description='A workaround to use jupyter instead of excel + tex for MIPT laboratory reports creation',
classifiers=[ 'License :: OSI Approved :: MIT License',
'Programming Language :: Python :: 3.6',
],
keywords='MIPT tex excel',
author='Andrei Pago',
author_email='andpago@gmail.com',
license='MIT',
packages=['excelmustdie'],
install_requires=['jupyter', 'pandas', 'matplotlib'],
include_package_data=True,
zip_safe=False) 



import os
import json
import re

from excelmustdie import text

def create_report(fname):
    os.chdir('images')
    for name in os.listdir():
        if len(name) > 4 and name[-4:] == '.png':
            name = name[:-4]
            os.system('convert {}.png {}.eps'.format(name, name))
            
    os.chdir('..')


    with open('report.tex', 'w') as O:
        O.write(text.preambule)

    out = ''
    
    try:
        with open('TITLE_TEXT.txt', 'r') as I:
            out = I.read()
    except:
        pass
        
    out += text.head_end
    
    with open(fname + '.ipynb') as I:
        encoder = json.JSONDecoder()
        for cell in encoder.decode(I.read())['cells']:
            if (cell['cell_type'] == 'markdown'):
                for line in cell['source']:
                    line = line.strip()
                    out += '\n'
                    
                    if line.startswith('# '):
                        out += '\section{' + line[2:] + '}'
                    elif line.startswith('## '):
                        out += '\subsection{' + line[2:] + '}'
                    else:
                        out += line
                
            elif cell['cell_type'] == 'code':
                for line in cell['source']:
                    if 'plot_named' in line:
                        pos = line.index('plot_named')
                        
                        endline = line[pos:]
                        
                        firstcav = endline.index("'")
                        lastcav = endline[firstcav+1:].index("'") + firstcav + 1
                        
                        name = endline[firstcav + 1: lastcav]
                        
                        out += '\includegraphics[width=1\linewidth]{' + name + '.eps}'
                    elif 'show_image' in line:
                        imshow_re = re.compile(r'\s*show_image\s*\(["\'](\w+)["\']\)')
                        name = imshow_re.match(line).groups()[0]
                        out += '\includegraphics[width=1\linewidth]{' + name + '.eps}'

    with open('report.tex', 'a') as O:
        O.write(out)
        
        O.write('\n')
        
        O.write(text.term)

    # report created

    os.system('texmaker report.tex')
    
if __name__ == "__main__":
    create_report()
